/** RPM SQL Upgrade Script - Version 6.0.8 to Version 6.0.9 **/
/** Run this script for each of your RPM SQL databases **/
/** Change the database name below - RPMDB - to the name of your actual RPM database before running the script **/
/** Click Execute to run the script **/

USE RPMDB
GO

-- 6032
-- Section 1
ALTER TABLE [Audit] ADD AuditAccountID int NULL
ALTER TABLE [AuditArchive] ADD AuditAccountID int NULL
ALTER TABLE [Audit] ADD AuditStatementID int NULL
ALTER TABLE [AuditArchive] ADD AuditStatementID int NULL
GO

-- Section 2
UPDATE [Audit] Set AuditAccountID = AuditDataTableID WHERE AuditDataTable = 'Account' and AuditAccountID Is Null
GO
UPDATE [Audit] 
SET [Audit].AuditAccountID = [AccountID]
FROM [Audit] INNER JOIN [Transaction] ON [Audit].AuditDataTableID = [Transaction].TransId  
WHERE [Audit].AuditEvent='Reconcile Removed' AND [Audit].AuditAccountID Is Null
GO
UPDATE [Audit] Set AuditEvent = 'Reconcile Unreconcile All' Where AuditEvent = 'Reset Reconciliation'
GO
UPDATE [Audit] SET Audit.AuditDataNew = Replace([AuditDataNew],'Statement Ending Date:','Statement Ending:') WHERE [Audit].AuditEvent='Reconcile Bank Account';
GO
UPDATE [AuditArchive] Set AuditAccountID = AuditDataTableID WHERE AuditDataTable = 'Account' and AuditAccountID Is Null
GO
UPDATE [AuditArchive] 
SET [AuditArchive].AuditAccountID = [AccountID]
FROM [AuditArchive] INNER JOIN [TransactionArchive] ON [AuditArchive].AuditDataTableID = [TransactionArchive].TransId  
WHERE [AuditArchive].AuditEvent='Reconcile Removed' AND [AuditArchive].AuditAccountID Is Null
GO


-- Section 3
CREATE TABLE [dbo].[ReconcileStatement](
	[ReconcileStatementID] [int] IDENTITY(1,1) NOT NULL,
	[AccountID] [int],
	[StatementDate] datetime NOT NULL,
	[OutstandingDate] datetime NULL,
	[EndingBalance] money NULL,
	[CurrentTrans] bit NULL,
	[ArchiveTrans] bit NULL,
	[LastUpdated] datetime NULL,
 CONSTRAINT [aaaaaReconcileStatement_PK] PRIMARY KEY NONCLUSTERED 
(
	[ReconcileStatementID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE UNIQUE NONCLUSTERED INDEX [AccountDate] ON [dbo].[ReconcileStatement] 
(
	[AccountID] ASC,
	[StatementDate] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
GO

-- Section 4
--Add statement dates from Archived Transaction
INSERT INTO ReconcileStatement ( AccountID, StatementDate, CurrentTrans, ArchiveTrans ) 
SELECT TransactionArchive.AccountID, TransactionArchive.ReconciledDate, 0, 0
FROM TransactionArchive 
GROUP BY TransactionArchive.AccountID, TransactionArchive.ReconciledDate 
HAVING (((TransactionArchive.ReconciledDate) Is Not Null));
GO
--Add statement dates from Transactions 
INSERT INTO ReconcileStatement ( AccountID, StatementDate , CurrentTrans, ArchiveTrans )
SELECT [Transaction].AccountID, [Transaction].ReconciledDate, 0, 0
FROM [Transaction] LEFT JOIN ReconcileStatement ON ([Transaction].AccountID = ReconcileStatement.AccountID) AND ([Transaction].ReconciledDate = ReconcileStatement.StatementDate)
WHERE (((ReconcileStatement.ReconcileStatementID) Is Null))
GROUP BY [Transaction].AccountID, [Transaction].ReconciledDate
HAVING ((([Transaction].ReconciledDate) Is Not Null));
GO
UPDATE ReconcileStatement Set OutstandingDate = StatementDate
GO
--Audit events ending balance
DECLARE @ReconcileStatementID INT
DECLARE @EndingBalanceAudit nvarchar(100)
DECLARE @AuditDate datetime
DECLARE TableCursor cursor for
SELECT ReconcileStatement.ReconcileStatementID, SUBSTRING([AuditDataNew], CHARINDEX('Ending Balance:',[AuditDataNew])+16, 100) AS EndingBalanceAudit, AuditDate
FROM ReconcileStatement INNER JOIN [Audit] ON ReconcileStatement.AccountID = [Audit].AuditAccountID 
WHERE [Audit].AuditEvent='Reconcile Bank Account' AND 
CHARINDEX(Format([StatementDate], 'M/d/yyyy') ,[AuditDataNew])>0 
ORDER BY [Audit].AuditDate DESC
Open TableCursor
Fetch next from TableCursor Into @ReconcileStatementID, @EndingBalanceAudit, @AuditDate
While @@FETCH_STATUS = 0
BEGIN
	UPDATE ReconcileStatement Set EndingBalance = Cast(Replace(Replace(@EndingBalanceAudit, '(', '-'), ')', '') AS Money), LastUpdated = @AuditDate Where ReconcileStatementID = @ReconcileStatementID and EndingBalance Is Null
	Fetch next from TableCursor Into @ReconcileStatementID, @EndingBalanceAudit, @AuditDate
END
Close TableCursor
Deallocate TableCursor
--Audit archive events ending balance
DECLARE TableCursor cursor for
SELECT ReconcileStatement.ReconcileStatementID, SUBSTRING([AuditDataNew], CHARINDEX('Ending Balance:',[AuditDataNew])+16, 100) AS EndingBalanceAudit, AuditDate
FROM ReconcileStatement INNER JOIN [AuditArchive] ON ReconcileStatement.AccountID = [AuditArchive].AuditAccountID 
WHERE [AuditArchive].AuditEvent='Reconcile Bank Account' AND 
CHARINDEX(Format([StatementDate], 'M/d/yyyy') ,[AuditDataNew])>0 
ORDER BY [AuditArchive].AuditDate DESC;
Open TableCursor
Fetch next from TableCursor Into @ReconcileStatementID, @EndingBalanceAudit, @AuditDate
While @@FETCH_STATUS = 0
BEGIN	
	UPDATE ReconcileStatement Set EndingBalance = Cast(Replace(Replace(@EndingBalanceAudit, '(', '-'), ')', '') AS money), LastUpdated = @AuditDate Where ReconcileStatementID = @ReconcileStatementID and EndingBalance Is Null
	Fetch next from TableCursor Into @ReconcileStatementID, @EndingBalanceAudit, @AuditDate
END
Close TableCursor
Deallocate TableCursor
--Update current flag for statements
UPDATE ReconcileStatement
SET ReconcileStatement.[CurrentTrans] = 1
FROM [Transaction] INNER JOIN ReconcileStatement ON ([Transaction].ReconciledDate = ReconcileStatement.StatementDate) AND ([Transaction].AccountID = ReconcileStatement.AccountID) 
GO
--Update archive flag for statements
UPDATE ReconcileStatement
SET ReconcileStatement.[ArchiveTrans] = 1
FROM [TransactionArchive] INNER JOIN ReconcileStatement ON ([TransactionArchive].ReconciledDate = ReconcileStatement.StatementDate) AND ([TransactionArchive].AccountID = ReconcileStatement.AccountID) 
GO
-- Update the last updated date for any remaining statements without an audit record
UPDATE ReconcileStatement SET LastUpdated = StatementDate Where LastUpdated Is Null
GO
--Add in process statements
INSERT INTO ReconcileStatement ( AccountID, StatementDate, OutstandingDate, EndingBalance, CurrentTrans, ArchiveTrans ) 
SELECT AccountList.AccountID, AccountList.StatementEndDate, AccountList.OutstandingDate, AccountList.EndingBalance, 1 as CurTrans, 0 as ArchTrans
FROM AccountList LEFT JOIN ReconcileStatement ON (AccountList.StatementEndDate = ReconcileStatement.StatementDate) AND (AccountList.AccountID = ReconcileStatement.AccountID) 
WHERE (((AccountList.StatementEndDate) Is Not Null) AND ((ReconcileStatement.ReconcileStatementID) Is Null));
GO
--Update the reconciled date for in process clearing items
UPDATE [Transaction]
SET [Transaction].ReconciledDate = [StatementEndDate]
FROM AccountList INNER JOIN [Transaction] ON AccountList.AccountId = [Transaction].AccountId  WHERE [Transaction].Reconciled=0 AND [Transaction].Reconciling=1 AND [Transaction].TransDate<=[OutstandingDate];
GO
--Remove any future clearing items
UPDATE [Transaction] Set Reconciling = 0 Where ReconciledDate Is Null
GO
-- create temporary view
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [dbo].[vwTemp_AuditDetailByStatementAcctDateUpdate]  AS 
SELECT [Audit].AuditID, [Audit].AuditAccountID, [Audit].AuditStatementID, 
(CASE WHEN CHARINDEX('Statement Ending', [AuditDataOld]) > 0 THEN SUBSTRING([AuditDataOld], Charindex('Statement Ending:', [AuditDataOld]) + 18, 1000) 
ELSE 
(CASE WHEN CHARINDEX('Statement Ending', [AuditItem])>0 THEN SUBSTRING([AuditItem], Charindex('Statement Ending:',[AuditItem]) + 18, 1000) 
ELSE 
(CASE WHEN CHARINDEX('Statement Ending',[AuditDataNew]) > 0 THEN 
(CASE WHEN CHARINDEX('Cleared Balance', [AuditDataNew]) > 0 THEN
SUBSTRING([AuditDataNew], CHARINDEX('Statement Ending', [AuditDataNew])+18, Charindex('Cleared Balance:', [AuditDataNew])-21) 
ELSE SUBSTRING([AuditDataNew], Charindex('Cleared Balance:', [AuditDataNew])+ 18, 1000) 
END) 
END)
END)
END) as StatementDate,
Charindex('Statement Ending:',[AuditItem]) AS Stmt_AuditItem, 
Charindex('Statement Ending:', [AuditDataNew]) AS Stmt_AuditNew, 
Charindex('Cleared Balance:', [AuditDataNew]) AS Clr_AuditNew, 
Charindex('Statement Ending:', [AuditDataOld]) AS Stmt_AuditOld
FROM [Audit] 
WHERE ((([Audit].AuditStatementID) Is Null) AND ((Charindex('Statement Ending:',[AuditItem]))>0) AND ((Left([AuditEvent],5))='Recon')) 
OR ((([Audit].AuditStatementID) Is Null) AND ((Charindex('Statement Ending:', [AuditDataNew]))>0) AND ((Left([AuditEvent],5))='Recon')) 
OR ((([Audit].AuditStatementID) Is Null) AND ((Charindex('Statement Ending:', [AuditDataOld]))>0) AND ((Left([AuditEvent],5))='Recon'))
GO
--Update statement id in audit events
DECLARE @ReconcileStatementID INT
DECLARE @AuditID INT
DECLARE TableCursor cursor for
SELECT vwTemp_AuditDetailByStatementAcctDateUpdate.AuditID, ReconcileStatement.ReconcileStatementID
FROM vwTemp_AuditDetailByStatementAcctDateUpdate INNER JOIN ReconcileStatement ON 
(vwTemp_AuditDetailByStatementAcctDateUpdate.StatementDate = ReconcileStatement.StatementDate) AND (vwTemp_AuditDetailByStatementAcctDateUpdate.AuditAccountID = ReconcileStatement.AccountID);
Open TableCursor
Fetch next from TableCursor Into @AuditID, @ReconcileStatementID
While @@FETCH_STATUS = 0
BEGIN	
	UPDATE [Audit] Set AuditStatementID = @ReconcileStatementID Where AuditID = @AuditID
	Fetch next from TableCursor Into @AuditID, @ReconcileStatementID
END
Close TableCursor
Deallocate TableCursor

-- Section 5
ALTER TABLE [AccountList] Add ReconcileStatementID Int
GO

--delete default constraint
DECLARE
    @constraint NVARCHAR(200),
    @command NVARCHAR(300)
SELECT
    @constraint = default_constraints.name
FROM 
    sys.all_columns
INNER JOIN
    sys.tables ON all_columns.object_id = tables.object_id
INNER JOIN 
    sys.schemas ON tables.schema_id = schemas.schema_id
INNER JOIN
    sys.default_constraints ON all_columns.default_object_id = default_constraints.object_id
WHERE 
    schemas.name = 'dbo'
AND
    tables.name = 'AccountList'
AND
    all_columns.name = 'EndingBalance'
IF @constraint IS NOT NULL
BEGIN
    SELECT @command = 'ALTER TABLE AccountList DROP CONSTRAINT ' + @constraint;
    EXECUTE(@command)
END
GO
ALTER Table [AccountList] Drop Column [EndingBalance]
GO
ALTER Table [AccountList] Drop Column [StatementEndDate]
GO
ALTER Table [AccountList] Drop Column [OutstandingDate]
GO
DELETE from [UserSecurity] Where ObjectID = 70
GO

--6078
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

ALTER VIEW [dbo].[vwSchdDepositActive]  AS 
SELECT SchedDeposit.SchedDepositID, SchedDeposit.ClientID, SchedDeposit.AccountID,
Client.ClientType, Client.SupportType,
SchedDeposit.TransType, SchedDeposit.Subcategory, SchedDeposit.DepositFreq, SchedDeposit.DepositAmt,
SchedDeposit.NextDepositDate, SchedDeposit.DepositEndDate, SchedDeposit.Notes, 
SchedDeposit.CreatedBy, SchedDeposit.LastUpdated, SchedDeposit.ProcessDate,
Client.ClientName, Client.AcctMgrID,
[AccountDesc] + ': ' + (CASE WHEN [AccountList].[ClientID] > 0 THEN 'Individual' ELSE 'Mass' END) AS AcctDescType
FROM ((SchedDeposit INNER JOIN Client ON SchedDeposit.ClientId = Client.ClientId) INNER JOIN 
(AccountList INNER JOIN ClientAccount ON AccountList.AccountID = ClientAccount.AccountID) 
ON (SchedDeposit.AccountID = ClientAccount.AccountID) AND (SchedDeposit.ClientId = ClientAccount.ClientID)) 
WHERE (((SchedDeposit.NextDepositDate) Is Not Null) 
AND ((Client.StatusType)='Active') AND ((ClientAccount.InactiveAcct)=0) AND ((AccountList.InactiveMass)=0));

GO

--6020
ALTER TABLE [Housing] ALTER COLUMN [HousingNote] nvarchar(max)
GO

--5937
ALTER TABLE [CheckLayout] ADD [AccountName] Bit Null
GO
ALTER TABLE [CheckLayout] ADD [AccountNameTop] Real Null
GO
ALTER TABLE [CheckLayout] ADD [AccountNameLeft] Real Null
GO
UPDATE [CheckLayout] Set [AccountName] = 0, [AccountNameTop] = 0, [AccountNameLeft] = 0
GO

--5514
INSERT INTO UserSecurity ( UserID, SecurityLevel, ObjectID ) SELECT UserSecurity.UserID, UserSecurity.SecurityLevel, 110 FROM UserSecurity WHERE UserSecurity.ObjectID = 65
INSERT INTO UserSecurity ( UserID, SecurityLevel, ObjectID ) SELECT UserSecurity.UserID, UserSecurity.SecurityLevel, 111 FROM UserSecurity WHERE UserSecurity.ObjectID = 65
INSERT INTO UserSecurity ( UserID, SecurityLevel, ObjectID ) SELECT UserSecurity.UserID, UserSecurity.SecurityLevel, 112 FROM UserSecurity WHERE UserSecurity.ObjectID = 65
INSERT INTO UserSecurity ( UserID, SecurityLevel, ObjectID ) SELECT UserSecurity.UserID, UserSecurity.SecurityLevel, 113 FROM UserSecurity WHERE UserSecurity.ObjectID = 65
GO
DELETE from [UserSecurity] Where ObjectID = 37
GO
UPDATE Audit Set AuditEvent = 'Client Fee Posting' Where AuditEvent = 'Client Fee Transactions'
UPDATE Audit Set AuditEvent = 'Client Balance Payout' Where AuditEvent = 'Client Balance Transactions'
UPDATE Audit Set AuditEvent = 'Reset Client Balance Payout' Where AuditEvent = 'Reset Client Balance'
UPDATE Audit Set AuditEvent = 'Reset Bank Fee Posting' Where AuditEvent = 'Reset Bank Fee'
UPDATE Audit Set AuditEvent = 'Reset Client Fee Posting' Where AuditEvent = 'Reset Client Fee'
GO

--6088
ALTER TABLE [ClientExpense] ALTER COLUMN [Category] nvarchar(50)
GO

-- 5686
ALTER TABLE [SchedTransfer] ADD [ToTransType] nvarchar(50)
ALTER TABLE [SchedTransfer] ADD [ToSubcategory] nvarchar(50)
GO

UPDATE [SchedTransfer] Set ToTransType = TransType, ToSubcategory = Subcategory
GO

-- 6042
ALTER TABLE [Income] Add [Subcategory] nvarchar(50)
ALTER TABLE [Income] Add [DepositFrom] nvarchar(50)
ALTER TABLE [Income] Add [IncomeNote] nvarchar(max)
GO
ALTER TABLE [ClientExpense] Add [Subcategory] nvarchar(50)
ALTER TABLE [ClientExpense] Add [PayTo] nvarchar(50)
ALTER TABLE [ClientExpense] Add [ExpenseNote] nvarchar(max)
GO

ALTER TABLE [dbo].[Income]  WITH CHECK ADD  CONSTRAINT [IncomeSubcategory_FK00] FOREIGN KEY([Subcategory])
REFERENCES [dbo].[TransSubcategoryList] ([Subcategory])
ON UPDATE CASCADE
GO
ALTER TABLE [dbo].[Income] CHECK CONSTRAINT [IncomeSubcategory_FK00]
GO
ALTER TABLE [dbo].[ClientExpense]  WITH CHECK ADD  CONSTRAINT [ClientExpenseSubcategory_FK00] FOREIGN KEY([Subcategory])
REFERENCES [dbo].[TransSubcategoryList] ([Subcategory])
ON UPDATE CASCADE
GO
ALTER TABLE [dbo].[ClientExpense] CHECK CONSTRAINT [ClientExpenseSubcategory_FK00]
GO

-- 6119
ALTER TABLE [SchedTransfer] Add [PaymentMethod] nvarchar(50)
ALTER TABLE [AccountList] Add [AcctType] smallint Null
ALTER TABLE [PaymentMethod] Add [SelectXfer] bit Null
ALTER TABLE [PaymentMethod] Add [SelectExp] bit Null
GO
UPDATE PaymentMethod Set SelectExp = 0
UPDATE PaymentMethod Set SelectXfer = 0
UPDATE PaymentMethod Set SelectExp = 1 Where PaymentMethod = 'XEFT'
UPDATE PaymentMethod Set SelectXfer = -1 Where PaymentMethod = 'Transfer Out'
GO
Insert Into PaymentMethod (PaymentMethod, AmtX, AmtXfer, SelectSchd, SelectInd, SelectRpt, SelectRecRange, Dep, Pmt, Trn, Bal, SelectXfer, SelectExp)
Values ('Export Transfer', 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1)
GO
UPDATE SchedTransfer Set PaymentMethod = 'Transfer' where PaymentMethod Is Null and ToAccountID = FromAccountID
UPDATE SchedTransfer Set PaymentMethod = 'Transfer Out' where PaymentMethod Is Null and ToAccountID <> FromAccountID
GO
UPDATE AccountList Set AcctType = 1 Where AcctType Is Null
GO

-- 6123
ALTER TABLE [PaymentMethod] Add [PmtMethodDesc] nvarchar(100)
GO
UPDATE PaymentMethod Set PmtMethodDesc = 'Money entering this account from another account' Where PaymentMethod = 'Transfer In'
UPDATE PaymentMethod Set PmtMethodDesc = 'A transfer transaction that can be exported to a file' Where PaymentMethod = 'Export Transfer'
UPDATE PaymentMethod Set PmtMethodDesc = 'Money moved between clients and/or categories within the same account' Where PaymentMethod = 'Transfer'
UPDATE PaymentMethod Set PmtMethodDesc = 'Money leaving this account to enter another account' Where PaymentMethod = 'Transfer Out'
UPDATE PaymentMethod Set PmtMethodDesc = 'Money deposited into this account' Where PaymentMethod = 'Deposit'
UPDATE PaymentMethod Set PmtMethodDesc = 'An ATM transaction or cash given to the client' Where PaymentMethod = 'ATM/Cash'
UPDATE PaymentMethod Set PmtMethodDesc = 'A check printed from RPM' Where PaymentMethod = 'Check'
UPDATE PaymentMethod Set PmtMethodDesc = 'A transaction made using a credit card' Where PaymentMethod = 'Credit Card'
UPDATE PaymentMethod Set PmtMethodDesc = 'A debit transaction (often used for adjustments, like bank fees)' Where PaymentMethod = 'Debit'
UPDATE PaymentMethod Set PmtMethodDesc = 'A transaction made using a debit card' Where PaymentMethod = 'Debit Card'
UPDATE PaymentMethod Set PmtMethodDesc = 'An electronic transaction (often used for automatically withdrawn funds) ' Where PaymentMethod = 'EFT'
UPDATE PaymentMethod Set PmtMethodDesc = 'A manually entered check (often used for handwritten or past checks)' Where PaymentMethod = 'Manual Check'
UPDATE PaymentMethod Set PmtMethodDesc = 'A check that is ready to be printed' Where PaymentMethod = 'Print'
UPDATE PaymentMethod Set PmtMethodDesc = 'An electronic transaction that can be exported to a file' Where PaymentMethod = 'XEFT'
GO

/****** Object:  View [dbo].[vwSchdTransferActive]    Script Date: 11/25/2024 3:30:33 PM ******/
DROP VIEW [dbo].[vwSchdTransferActive]
GO

/****** Object:  View [dbo].[vwSchdTransferActive]    Script Date: 11/25/2024 3:30:33 PM ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [dbo].[vwSchdTransferActive]  AS 
SELECT SchedTransfer.*, Client.ClientNameFL as ToClientNameFL, Client_1.ClientNameFL as FromClientNameFL
FROM (((((SchedTransfer INNER JOIN Client ON SchedTransfer.FromClientID = Client.ClientId) INNER JOIN Client AS Client_1 ON SchedTransfer.ToClientID = Client_1.ClientId) 
INNER JOIN ClientAccount ON (SchedTransfer.FromAccountID = ClientAccount.AccountID) AND (SchedTransfer.FromClientID = ClientAccount.ClientID)) 
INNER JOIN ClientAccount AS ClientAccount_1 ON (SchedTransfer.ToAccountID = ClientAccount_1.AccountID) AND (SchedTransfer.ToClientID = ClientAccount_1.ClientID)) 
INNER JOIN AccountList ON ClientAccount.AccountID = AccountList.AccountID) INNER JOIN AccountList AS AccountList_1 ON ClientAccount_1.AccountID = AccountList_1.AccountID
WHERE (((SchedTransfer.NextTransferDate) Is Not Null) AND ((Client.StatusType)='Active') AND ((Client_1.StatusType)='Active') AND ((ClientAccount.InactiveAcct)=0) 
AND ((AccountList.InactiveMass)=0) AND ((ClientAccount_1.InactiveAcct)=0) AND ((AccountList_1.InactiveMass)=0));

GO

-- Delete error messages more than 3 years old
DELETE FROM ErrorMessages WHERE Error_Date < Dateadd(yyyy, -3, GETDATE())
GO

-- Update version
Update VersionDataSQL Set CurrentVersionNum = 609
GO

-- Add audit event for upgrade
Insert Into [Audit] (AuditDate, UserID, AuditEvent, AuditItem) Values (GETDATE(), 'Upgrade', 'Upgrade Installed', '6.0.9')
GO
